Bonny.Naming (bonny v1.5.0)

Copy Markdown View Source

Naming functions

Summary

Functions

Converts an elixir module name to a string for use as the CRD's kind.

Extract the CRD API version from the module name. Defaults to "v1"

Functions

module_to_kind(mod)

@spec module_to_kind(atom()) :: String.t()

Converts an elixir module name to a string for use as the CRD's kind.

Examples

iex> Bonny.Naming.module_to_kind(Pod)
"Pod"

iex> Bonny.Naming.module_to_kind(Controllers.V1.Pod)
"Pod"

module_version(mod)

@spec module_version(atom()) :: String.t()

Extract the CRD API version from the module name. Defaults to "v1"

Examples

iex> Bonny.Naming.module_version(Pod)
"v1"

iex> Bonny.Naming.module_version(Controllers.V1.Pod)
"v1"

iex> Bonny.Naming.module_version(Controllers.V1Alpha1.Pod)
"v1alpha1"