View Source Bonny.Pluggable.Logger (bonny v1.0.0-rc.0)
A pluggable step for logging basic action event information in the format:
{:add, "example.com/v1", "Widget"} - Processing event
{:add, "example.com/v1", "Widget"} - Status applied
{:add, "example.com/v1", "Widget"} - Normal event emitted
{:add, "example.com/v1", "Widget"} - Descendant {"v1", "Deployment", "default/nginx"} appliedTo use it, just add a step to the desired module.
step Bonny.Pluggable.Logger, level: :debug
options
Options
:level- The log level at which this plug should log its request info. Default is:info. The list of supported levels is available in theLoggerdocumentation.
Link to this section Summary
Functions
Callback implementation for Pluggable.call/2.
Callback implementation for Pluggable.init/1.
Link to this section Functions
Callback implementation for Pluggable.call/2.
Callback implementation for Pluggable.init/1.